<?php
namespace Drupal\miniorange_2fa\form;

use Drupal\Core\Url;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\miniorange_2fa\MoAuthUtilities;
use Drupal\miniorange_2fa\AuthenticationType;
use Drupal\Component\Render\FormattableMarkup;

class UserMfaSetup extends FormBase {
    public function getFormId() {
        return "mo_mfa_form";
    }

    public function buildForm(array $form, FormStateInterface $form_state) {
        $userId = \Drupal::currentUser()->id();
        $variables_and_values = array(
            'mo_auth_enable_two_factor',
            'mo_auth_2fa_kba_questions',
            'mo_auth_2fa_allow_reconfigure_2fa',
            'mo_auth_firstuser_id'
        );
        $mo_db_values = MoAuthUtilities::miniOrange_set_get_configurations( $variables_and_values, 'GET' );

        $custom_attribute = MoAuthUtilities::get_users_custom_attribute(\Drupal::currentUser()->id());

        if( isset( $custom_attribute[0]->activated_auth_methods ) ) {
            $authMethod = $custom_attribute[0]->activated_auth_methods;
            $form['mo_2fa_method'] = array(
                '#type' => 'fieldset',
            );

            $form['mo_2fa_method']['mo_configured_2fa_methods'] = array(
                '#markup' => t('<strong>Configured 2FA method: </strong>' . $authMethod . ' ')
            );

            if( $mo_db_values['mo_auth_2fa_allow_reconfigure_2fa'] == 'Allowed' && isset( $mo_db_values['mo_auth_firstuser_id'] ) && $mo_db_values['mo_auth_firstuser_id'] != $userId ) {
                $form['mo_2fa_method']['mo_configure_reconfigure_button'] = array(
                    '#type' => 'link',
                    '#title' => $this->t('Re-configure'),
                    '#url' => Url::fromRoute('miniorange_2fa.re_configure'),
                    '#attributes' => [
                        'class' => [
                            'use-ajax',
                            'button',
                        ],
                    ],
                );
            }


            if( $authMethod == AuthenticationType::$SMS['code'] || $authMethod == AuthenticationType::$SMS_AND_EMAIL['code'] || $authMethod == AuthenticationType::$OTP_OVER_PHONE['code'] ){
                $form['mo_2fa_method']['mo_configured_phone_number'] = array(
                    '#markup' => t('<br><br><strong>Configured phone number: </strong>' . $custom_attribute[0]->phone_number . '<br><br>')
                );
                //TODO: add feature to change phone number
            }
            /**
             * Configure Backup 2FA method section
             */
            if( $mo_db_values['mo_auth_2fa_kba_questions'] == 'Allowed' ) {
                $configuredMethods = MoAuthUtilities::mo_auth_get_configured_methods($custom_attribute);
                $moTitle = 'Configure';
                if ( in_array( AuthenticationType::$KBA['code'], $configuredMethods ) ) {
                    $moTitle = 'Re-Configure';
                }
                if ( $authMethod != AuthenticationType::$KBA['code'] ) {
                    $form['mo_2fa_method']['mo_configure_backup_2fa'] = array(
                        '#markup' => t('<br><strong>Configure backup 2FA method security questions (KBA):</strong> ')
                    );
                    $form['mo_2fa_method']['mo_configure_backup_2fa_button'] = array(
                        '#type' => 'link',
                        '#title' => $this->t($moTitle),
                        '#url' => Url::fromRoute('miniorange_2fa.configure_kba'),
                        '#attributes' => [
                            'class' => [
                                'use-ajax',
                                'button',
                            ],
                        ],
                    );
                }
            }
            
            if( in_array( $authMethod, MoAuthUtilities::mo_TOTP_2fa_mentods() ) && isset( $custom_attribute[0]->qr_code_string ) && !empty( $custom_attribute[0]->qr_code_string ) ) {
                $qrCode = $custom_attribute[0]->qr_code_string;
                $image = new FormattableMarkup('<img src="data:image/jpg;base64, ' . $qrCode . '"/>', [':src' => $qrCode]);

                $form['mo_2fa_method']['mo_qr_code_details'] = array(
                    '#markup' => t('<br><ul><li>You can scan the below QR Code from ' . ucwords( strtolower( $authMethod ) ) . ' app to configure the 2FA on multiple devices.</li></ul>')
                );

                $form['mo_2fa_method']['mo_scan_qr_code_google_authenticator'] = array(
                    '#markup' => $image = isset($image) ? $image : '',
                );
            }
        }

        /**
         * We can't let you see the settings if you are not admin and trying to edit 2FA for some other user
         */
        if( !$mo_db_values['mo_auth_enable_two_factor'] || !MoAuthUtilities::isUserCanSee2FASettings() ) {
            return $form;
        }

        $form['mo_2fa_method']['mo_mfa_enable'] = array(
            '#type'=> 'checkbox',
            '#title'=> t('Enable MFA for your account'),
            '#default_value'=> isset( $custom_attribute[0]->enabled ) ? $custom_attribute[0]->enabled : FALSE,
            '#description'=> t('<strong>Note:</strong> You will not be prompted for 2FA if you disable this'),
            '#prefix' => '<hr>'
        );

        $form['mo_2fa_method']['mo_mfa_form_save'] = array(
            '#type'=>'submit',
            '#value'=> t('Submit'),
            '#button_type' => 'primary',
        );

        return $form;
    }

    public function submitForm(array &$form, FormStateInterface $form_state) {
        $formValues = $form_state->getValues();
        if( $formValues["mo_mfa_enable"] === 0 ) {
            if( !MoAuthUtilities::isSkipNotAllowed() ) {
                \Drupal::messenger()->addError(t("You are not allowed to disable 2FA. Please contact your site administrator"));
                return;
            }
        }
        MoAuthUtilities::updateMfaSettingsForUser(\Drupal::currentUser()->id(), $formValues["mo_mfa_enable"] == 1 ? 1 : 0);
        \Drupal::messenger()->addStatus(t('2FA settings are updated for your account'));
    }
}